@@ -5,11 +5,14 @@ from decorator import decorator
 from paste.deploy.converters import asbool
 from pylons.decorators.cache import beaker_cache, create_cache_key, _make_dict_from_args
 from pylons.decorators.util import get_pylons
+from pylons.controllers.util import etag_cache as pylons_etag_cache
 import pylons.config
 
 __all__ = ["ckan_cache", "get_cache_expires"]
 
-cache_enabled = asbool(pylons.config.get("cache_enabled", "False"))
+cache_enabled = asbool(pylons.config.get('cache_enabled', 'False')) or \
+                asbool(pylons.config.get('ckan.cache_enabled', 'False'))
+cache_validation_enabled = asbool(pylons.config.get('ckan.cache_validation_enabled', 'True'))
 
 def ckan_cache(test=lambda *av, **kw: 0,
           key="cache_default",
@@ -206,3 +209,6 @@ def get_cache_expires(module_or_func):
     cache_expires = int(pylons.config.get(cfg_expires, default_expires))
     return cache_expires
 
+def etag_cache(page_hash):
+    if cache_validation_enabled:
+        pylons_etag_cache(page_hash)
