@@ -1,11 +1,17 @@
 import logging
 import re
+import datetime
 
 import ckan.authz
 from ckan.plugins import PluginImplementations, IGroupController, IPackageController
 from ckan.logic import NotFound, check_access, NotAuthorized, ValidationError
 from ckan.lib.base import _
-from ckan.lib.dictization.model_dictize import group_dictize, package_dictize
+from ckan.lib.dictization.model_dictize import (package_dictize,
+                                                package_to_api1,
+                                                package_to_api2,
+                                                group_dictize,
+                                                group_to_api1,
+                                                group_to_api2)
 from ckan.lib.dictization.model_save import (group_api_to_dict,
                                              package_api_to_dict,
                                              group_dict_save,
@@ -70,8 +76,77 @@ def check_group_auth(data_dict, context):
     for group in groups:
         check_access(group, model.Action.EDIT, context)
 
-def package_update(data_dict, context):
+def _make_latest_rev_active(context, q):
 
+    session = context['model'].Session
+
+    old_current = q.filter_by(current=True).first()
+    if old_current:
+        old_current.current = False
+        session.add(old_current)
+
+    latest_rev = q.filter_by(expired_timestamp=datetime.datetime(9999, 12, 31)).one()
+    latest_rev.current = True
+    if latest_rev.state in ('pending-deleted', 'deleted'):
+        latest_rev.state = 'deleted'
+    else:
+        latest_rev.state = 'active'
+
+    session.add(latest_rev)
+        
+    ##this is just a way to get the latest revision that changed
+    ##in order to timestamp
+    old_latest = context.get('latest_revision_date')
+    if old_latest:
+        if latest_rev.revision_timestamp > old_latest:
+            context['latest_revision_date'] = latest_rev.revision_timestamp
+            context['latest_revision'] = latest_rev.revision_id
+    else:
+        context['latest_revision_date'] = latest_rev.revision_timestamp
+        context['latest_revision'] = latest_rev.revision_id
+
+def make_latest_pending_package_active(context):
+
+    model = context['model']
+    session = model.Session
+    id = context["id"]
+    pkg = model.Package.get(id)
+
+    check_access(pkg, model.Action.EDIT, context)
+
+    #packages
+    q = session.query(model.PackageRevision).filter_by(id=pkg.id)
+    _make_latest_rev_active(context, q)
+
+    #resources
+    for resource in pkg.resource_groups_all[0].resources_all:
+        q = session.query(model.ResourceRevision).filter_by(id=resource.id)
+        _make_latest_rev_active(context, q)
+
+    #tags
+    for tag in pkg.package_tag_all:
+        q = session.query(model.PackageTagRevision).filter_by(id=tag.id)
+        _make_latest_rev_active(context, q)
+
+    #extras
+    for extra in pkg.extras_list:
+        q = session.query(model.PackageExtraRevision).filter_by(id=extra.id)
+        _make_latest_rev_active(context, q)
+
+    latest_revision = context.get('latest_revision')
+    if not latest_revision:
+        return
+
+    q = session.query(model.Revision).filter_by(id=latest_revision)
+    revision = q.first()
+    revision.approved_timestamp = datetime.datetime.now()
+    session.add(revision)
+    
+    session.commit()        
+    session.remove()        
+
+
+def package_update(data_dict, context):
     model = context['model']
     user = context['user']
     id = context["id"]
@@ -84,12 +159,14 @@ def package_update(data_dict, context):
 
     if pkg is None:
         raise NotFound(_('Package was not found.'))
+    context["id"] = pkg.id
 
     check_access(pkg, model.Action.EDIT, context)
-    check_group_auth(data_dict, context)
 
     data, errors = validate(data_dict, schema, context)
 
+    check_group_auth(data, context)
+
     if errors:
         model.Session.rollback()
         raise ValidationError(errors, package_error_summary(errors))
@@ -100,7 +177,7 @@ def package_update(data_dict, context):
         if 'message' in context:
             rev.message = context['message']
         else:
-            rev.message = _(u'REST API: Create object %s') % data.get("name")
+            rev.message = _(u'REST API: Update object %s') % data.get("name")
 
     pkg = package_dict_save(data, context)
 
@@ -108,8 +185,8 @@ def package_update(data_dict, context):
         for item in PluginImplementations(IPackageController):
             item.edit(pkg)
         model.repo.commit()        
-
-    return package_dictize(pkg, context)
+        return package_dictize(pkg, context)
+    return data
 
 
 def _update_package_relationship(relationship, comment, context):
@@ -159,8 +236,6 @@ def package_relationship_update(data_dict, context):
     comment = data_dict.get('comment', u'')
     return _update_package_relationship(entity, comment, context)
 
-
-
 def group_update(data_dict, context):
 
     model = context['model']
@@ -186,7 +261,7 @@ def group_update(data_dict, context):
     if 'message' in context:
         rev.message = context['message']
     else:
-        rev.message = _(u'REST API: Create object %s') % data.get("name")
+        rev.message = _(u'REST API: Update object %s') % data.get("name")
 
     group = group_dict_save(data, context)
 
@@ -205,19 +280,38 @@ def package_update_rest(data_dict, context):
 
     model = context['model']
     id = context["id"]
+    api = context.get('api_version') or '1'
     pkg = model.Package.get(id)
     context["package"] = pkg
     context["allow_partial_update"] = True
     dictized_package = package_api_to_dict(data_dict, context)
-    return package_update(dictized_package, context)
+    dictized_after = package_update(dictized_package, context)
+
+    pkg = context['package']
+
+    if api == '1':
+        package_dict = package_to_api1(pkg, context)
+    else:
+        package_dict = package_to_api2(pkg, context)
+
+    return package_dict
 
 def group_update_rest(data_dict, context):
 
     model = context['model']
     id = context["id"]
+    api = context.get('api_version') or '1'
     group = model.Group.get(id)
     context["group"] = group
     context["allow_partial_update"] = True
     dictized_package = group_api_to_dict(data_dict, context)
-    return group_update(dictized_package, context)
+    dictized_after = group_update(dictized_package, context)
 
+    group = context['group']
+
+    if api == '1':
+        group_dict = group_to_api1(group, context)
+    else:
+        group_dict = group_to_api2(group, context)
+
+    return group_dict
