@@ -38,6 +38,7 @@ def default_resource_schema():
         'hash': [ignore_missing, unicode],
         'state': [ignore],
         'position': [ignore],
+        'revision_timestamp': [ignore],
         '__extras': [ignore_missing, extras_unicode_convert, keep_extras],
     }
 
@@ -51,6 +52,8 @@ def default_tags_schema():
                  tag_length_validator,
                  tag_name_validator,
                  tag_not_uppercase],
+        'revision_timestamp': [ignore],
+        'state': [ignore],
     }
     return schema
 
@@ -164,9 +167,10 @@ def default_extras_schema():
     schema = {
         'id': [ignore],
         'key': [not_empty, unicode],
-        'value': [not_missing, unicode],
+        'value': [not_missing],
         'state': [ignore],
         'deleted': [ignore_missing],
+        'revision_timestamp': [ignore],
     }
     return schema
 
