@@ -40,7 +40,7 @@ class TestSearch(TestController):
 
     @classmethod
     def teardown_class(self):
-        CreateTestData.delete()
+        model.repo.rebuild_db()
 
     def _pkg_names(self, result):
         return ' '.join(result['results'])
@@ -99,6 +99,11 @@ class TestSearch(TestController):
         result = self.backend.query_for(model.Package).run(query=u'Expenditure Government China')
         assert len(result['results']) == 0, self._pkg_names(result)
 
+    def test_3_licence(self):
+        ## this should result, but it is here to check that at least it does not error
+        result = self.backend.query_for(model.Package).run(query=u'license:"OKD::Other (PublicsDomain)"')
+        assert result['count'] == 0, result
+
 # Quotation not supported now
 ##        # multiple words quoted
 ##        result = Search().search(u'"Government Expenditure"')
@@ -282,7 +287,7 @@ class TestSearchOverall(TestController):
 
     @classmethod
     def teardown_class(self):
-        CreateTestData.delete()
+        model.repo.rebuild_db()
 
     def _check_search_results(self, terms, expected_count, expected_packages=[], only_open=False, only_downloadable=False):
         options = QueryOptions()
@@ -334,7 +339,7 @@ class TestGeographicCoverage(TestController):
 
     @classmethod
     def teardown_class(self):
-        CreateTestData.delete()
+        model.repo.rebuild_db()
     
     def _do_search(self, q, expected_pkgs, count=None):
         options = QueryOptions()
@@ -389,7 +394,7 @@ class TestExtraFields(TestController):
 
     @classmethod
     def teardown_class(self):
-        CreateTestData.delete()
+        model.repo.rebuild_db()
     
     def _do_search(self, department, expected_pkgs, count=None):
         result = self.backend.query_for(model.Package).run(fields={'department':department})
@@ -424,8 +429,7 @@ class TestRank(TestController):
 
     @classmethod
     def teardown_class(self):
-        self.purge_all_packages()
-        CreateTestData.delete()
+        model.repo.rebuild_db()
     
     def _do_search(self, q, wanted_results):
         options = QueryOptions()
